/*
 * Copyright [2022] [https://www.baiduc.com]
 *
 * Snowy采用APACHE LICENSE 2.0开源协议，您在使用过程中，需要注意以下几点：
 *
 * 1.请不要删除和修改根目录下的LICENSE文件。
 * 2.请不要删除和修改Snowy源码头部的版权声明。
 * 3.本项目代码可免费商业使用，商业使用请保留源码和相关描述文件的项目出处，作者声明等。
 * 4.分发源码时候，请注明软件出处 https://www.baiduc.com
 * 5.不可二次分发开源参与同类竞品，如有想法可联系团队xiaonuobase@qq.com商议合作。
 * 6.若您的项目无法满足以上几点，需要更多功能代码，获取Snowy商业授权许可，请在官网购买授权，地址为 https://www.baiduc.com
 */
package vip.xiaonuo.erp.controller;

import cn.dev33.satoken.annotation.SaCheckPermission;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import com.github.xiaoymin.knife4j.annotations.ApiSupport;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RestController;
import vip.xiaonuo.common.annotation.CommonLog;
import vip.xiaonuo.common.pojo.CommonResult;
import vip.xiaonuo.common.pojo.CommonValidList;
import vip.xiaonuo.erp.entity.ErpBdMaterial;
import vip.xiaonuo.erp.param.*;
import vip.xiaonuo.erp.service.ErpBdMaterialService;

import javax.annotation.Resource;
import javax.validation.Valid;
import javax.validation.constraints.NotEmpty;
import java.util.List;

/**
 * erp_bd_material控制器
 *
 * @author 1
 * @date  2025/03/01 14:50
 */
@Api(tags = "erp_bd_material控制器")
@ApiSupport(author = "SNOWY_TEAM", order = 1)
@RestController
@Validated
public class ErpBdMaterialController {

    @Resource
    private ErpBdMaterialService erpBdMaterialService;

    /**
     * 获取erp_bd_material分页
     *
     * @author 1
     * @date  2025/03/01 14:50
     */
    @ApiOperationSupport(order = 1)
    @ApiOperation("获取erp_bd_material分页")
    @PostMapping("/biz/bdmaterial/page")
    public CommonResult<Page<ErpBdMaterial>> page(@RequestBody  ErpBdMaterialPageParam erpBdMaterialPageParam) {
        return CommonResult.data(erpBdMaterialService.page(erpBdMaterialPageParam));
    }

    /**
     * 添加erp_bd_material
     *
     * @author 1
     * @date  2025/03/01 14:50
     */
//    @ApiOperationSupport(order = 2)
//    @ApiOperation("添加erp_bd_material")
//    @CommonLog("添加erp_bd_material")
//    @SaCheckPermission("/biz/bdmaterial/add")
//    @PostMapping("/biz/bdmaterial/add")
//    public CommonResult<String> add(@RequestBody @Valid ErpBdMaterialAddParam erpBdMaterialAddParam) {
//        erpBdMaterialService.add(erpBdMaterialAddParam);
//        return CommonResult.ok();
//    }

    /**
     * 编辑erp_bd_material
     *
     * @author 1
     * @date  2025/03/01 14:50
     */
    @ApiOperationSupport(order = 3)
    @ApiOperation("编辑erp_bd_material")
    @PostMapping("/biz/bdmaterial/edit")
    public CommonResult<String> edit(@RequestBody @Valid ErpBdMaterialEditParam erpBdMaterialEditParam) {
        erpBdMaterialService.edit(erpBdMaterialEditParam);
        return CommonResult.ok();
    }

    /**
     * 删除erp_bd_material
     *
     * @author 1
     * @date  2025/03/01 14:50
     */
//    @ApiOperationSupport(order = 4)
//    @ApiOperation("删除erp_bd_material")
//    @PostMapping("/biz/bdmaterial/delete")
//    public CommonResult<String> delete(@RequestBody
//                                                   ErpBdMaterialIdParam erpBdMaterialIdParamList) {
//        erpBdMaterialService.delete(erpBdMaterialIdParamList);
//        return CommonResult.ok();
//    }

    /**
     * 获取erp_bd_material详情
     *
     * @author 1
     * @date  2025/03/01 14:50
     */
    @ApiOperationSupport(order = 5)
    @ApiOperation("获取erp_bd_material详情")
    @PostMapping("/biz/bdmaterial/detail")
    public CommonResult<ErpBdMaterial> detail(@RequestBody  ErpBdMaterialIdParam erpBdMaterialIdParam) {
        return CommonResult.data(erpBdMaterialService.detail(erpBdMaterialIdParam));
    }

    @ApiOperationSupport(order = 5)
    @ApiOperation("根据父id查询")
    @PostMapping("/biz/bdmaterial/getByParentId")
    public CommonResult<List<ErpBdMaterial>> getByParentId(@RequestBody ErpBdMaterialParentParam erpBdMaterial) {
        return CommonResult.data(erpBdMaterialService.getByParentId(erpBdMaterial));
    }


    @ApiOperationSupport(order = 5)
    @ApiOperation("根据树状")
    @PostMapping("/biz/bdmaterial/getTreeParentId")
    public CommonResult<List<ErpBdMaterialVo>> getTreeParentId(@RequestBody ErpBdMaterialParentParam erpBdMaterial) {
        return CommonResult.data(erpBdMaterialService.getTreeParentId(erpBdMaterial));
    }
    @ApiOperationSupport(order = 5)
    @ApiOperation("同步")
    @PostMapping("/biz/bdmaterial/sync")
    public CommonResult<String> sync() {
        erpBdMaterialService.sync();
        return CommonResult.ok();
    }


}
